// Assert functions return if executed without errors

#define _GNU_SOURCE

#include "libc.h"

#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <bestiola/assert.h>

char *assert_aprintf(const char * restrict fmt, ...) {
	char *strp;
	va_list ap;
	va_start(ap, fmt);
	int ret = vasprintf(&strp, fmt, ap);
	va_end(ap);
	if (ret == -1)
		assert_error_handler();
	return strp;
}

void assert_chdir(const char *path) {
	if (chdir(path))
		assert_error_handler();
}

void assert_close(int fd) {
	if (close(fd))
		assert_error_handler();
}

void assert_dup2(int oldfd, int newfd) {
	if (dup2(oldfd, newfd) == -1)
		assert_error_handler();
}

void assert_execvp(const char *file, char * const argv[]) {
	execvp(file, argv);
	assert_error_handler();
}

void assert_fchmod(int fd, mode_t mode) {
	if (fchmod(fd, mode))
		assert_error_handler();
}

void assert_fclose(FILE *stream) {
	if (fclose(stream))
		assert_error_handler();
}

int assert_fcntl(int fd, int cmd, ...) {
	switch (cmd) {
	case F_GETFD: {
		int flags = fcntl(fd, cmd);
		if (flags == -1)
			assert_error_handler();
		return flags;
	}
	case F_SETFD: {
		va_list ap;
		va_start(ap, cmd);
		if (fcntl(fd, cmd, va_arg(ap, int)))
			assert_error_handler();
		va_end(ap);
		return 0;
	}
	case F_GETFL: {
		int flags = fcntl(fd, cmd);
		if (flags == -1)
			assert_error_handler();
		return flags;
	}
	case F_SETFL: {
		va_list ap;
		va_start(ap, cmd);
		if (fcntl(fd, cmd, va_arg(ap, int)))
			assert_error_handler();
		va_end(ap);
		return 0;
	}
	default:
		assert_error_handler();
		return -1;
	}
}

FILE *assert_fdopen(int fd, const char *mode) {
	FILE *stream = fdopen(fd, mode);
	if (!stream)
		assert_error_handler();
	return stream;
}

void assert_fflush(FILE *stream) {
	if (fflush(stream))
		assert_error_handler();
}

FILE *assert_fopen(const char * restrict path, const char * restrict mode) {
	FILE *stream = fopen(path, mode);
	if (!stream)
		assert_error_handler();
	return stream;
}

void assert_fprintf(FILE * restrict stream, const char * restrict format, ...) {
	va_list ap;
	va_start(ap, format);
	int ret = vfprintf(stream, format, ap);
	va_end(ap);
	if (ret < 0)
		assert_error_handler();
}

void assert_fputs(const char * restrict s, FILE * restrict stream) {
	if (fputs(s, stream) == EOF)
		assert_error_handler();
}

size_t assert_fread(void * restrict ptr, size_t size, size_t nmemb,
	FILE * restrict stream) {
	size_t nread = fread(ptr, size, nmemb, stream);
	if (nread != nmemb && ferror(stream))
		assert_error_handler();
	return nread;
}

void assert_fread_noeof(void * restrict ptr, size_t size, size_t nmemb,
	FILE * restrict stream) {
	if (fread(ptr, size, nmemb, stream) != nmemb)
		assert_error_handler();
}

void assert_fstat(int fd, struct stat *buf) {
	if (fstat(fd, buf))
		assert_error_handler();
}

void assert_futimens(int fd, const struct timespec times[2]) {
	if (futimens(fd, times))
		assert_error_handler();
}

void assert_fwrite(const void * restrict ptr, size_t size, size_t nmemb,
	FILE * restrict stream) {
	if (fwrite(ptr, size, nmemb, stream) != nmemb)
		assert_error_handler();
}

int assert_getc(FILE *stream) {
	int c = getc(stream);
	if (c == EOF && ferror(stream))
		assert_error_handler();
	return c;
}

int assert_getchar(void) {
	int c = getchar();
	if (c == EOF && ferror(stdin))
		assert_error_handler();
	return c;
}

int assert_getchar_noeof(void) {
	int c = getchar();
	if (c == EOF)
		assert_error_handler();
	return c;
}

void *assert_malloc(size_t size) {
	void *ptr = malloc(size);
	if (!ptr && size)
		assert_error_handler();
	return ptr;
}

int assert_open(const char *path, int flags, ...) {
	int fd;
	if (flags & (O_CREAT | O_TMPFILE)) {
		va_list ap;
		va_start(ap, flags);
		fd = open(path, flags, va_arg(ap, mode_t));
		va_end(ap);
	}
	else
		fd = open(path, flags);
	if (fd == -1)
		assert_error_handler();
	return fd;
}

int assert_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
	int ret = poll(fds, nfds, timeout);
	if (ret == -1)
		assert_error_handler();
	return ret;
}

void assert_printf(const char * restrict format, ...) {
	va_list ap;
	va_start(ap, format);
	int ret = vprintf(format, ap);
	va_end(ap);
	if (ret < 0)
		assert_error_handler();
}

void assert_putc(int c, FILE *stream) {
	if (putc(c, stream) == EOF)
		assert_error_handler();
}

void assert_putchar(int c) {
	if (putchar(c) == EOF)
		assert_error_handler();
}

void assert_puts(const char *s) {
	if (puts(s) == EOF)
		assert_error_handler();
}

void *assert_realloc(void *ptr, size_t size) {
	ptr = realloc(ptr, size);
	if (!ptr && size)
		assert_error_handler();
	return ptr;
}

char *assert_strdup(const char *s) {
	char *sdup = strdup(s);
	if (!sdup)
		assert_error_handler();
	return sdup;
}

char *assert_strndup(const char *s, size_t n) {
	char *sdup = strndup(s, n);
	if (!sdup)
		assert_error_handler();
	return sdup;
}

unsigned long int assert_strtoul(const char *nptr, int base) {
	char *end;
	errno = 0;
	unsigned long int ul = strtoul(nptr, &end, base);
	if (errno || (*nptr && *end))
		assert_error_handler();
	return ul;
}
